<?php
if (!defined('ABSPATH')) {
    exit;
}

class Eh_Amazon_Pay_Hooks {

    protected $eh_amazon_pay_options;
    protected $id;
    protected $token;
    protected $eh_amazon;

    public function __construct() {
        $this->eh_amazon_pay_options = get_option('woocommerce_eh_amazon_pay_settings');
        if ($this->eh_amazon_pay_options['enabled'] === 'yes') {
            $this->eh_amazon = Eh_Amazon_Static::get_enable_amazon();
            $this->token = Eh_Amazon_Static::get_acc_token($this->eh_amazon_pay_options['ssl_mode']);
            $this->id = Eh_Amazon_Static::get_ref_id();
            add_action('wp_head', array($this, 'login_app_init'));
            add_action('wp', array($this, 'unset_amazon'));
            add_action('woocommerce_before_checkout_form', array($this, 'add_amazon_pay_button'));
            add_action('woocommerce_checkout_before_customer_details', array($this, 'add_amazon_address_widget_section'));
            add_action('woocommerce_checkout_before_customer_details', array($this, 'add_amazon_payment_widget_section'));
            add_action('woocommerce_available_payment_gateways', array($this, 'gateways_hide_on_review'));
            add_action('woocommerce_review_order_after_payment', array($this, 'add_policy_notes'));
            add_action('woocommerce_review_order_after_submit', array($this, 'add_cancel_order_elements'));
            add_action('woocommerce_checkout_update_order_review', array($this, 'update_check_info'));
            add_action( 'woocommerce_checkout_init', array( $this, 'remove_checkout_fields' ) );
        }
    }

    public function login_app_init() {
        if ('https' === $this->eh_amazon_pay_options['ssl_mode']) {
            ?>
            <script>
                window.onAmazonLoginReady = function () {
                    amazon.Login.setClientId("<?php echo esc_js($this->eh_amazon_pay_options['client_id']); ?>");
                    amazon.Login.setUseCookie(true);
                };
            </script>
            <?php
        }
    }

    public function unset_amazon() {
        if (( isset($_REQUEST['cancel_amazon_checkout']) && ($_REQUEST['cancel_amazon_checkout'] === 'true'))) {
            if (isset($_COOKIE["eh_amazon_access_token"])) {
                unset($_COOKIE["eh_amazon_access_token"]);
                setcookie("eh_amazon_access_token", '', time() - 3600);
            }
        }
    }

    public function add_amazon_pay_button() {
        if ($this->id === '' && $this->token === '' && !$this->eh_amazon && $this->eh_amazon_pay_options['amazon_on_checkout_page'] === 'yes') {
            $pwa_button_output = '<center><div class="eh_pay_with_amazon_button"><div class="eh_pay_with_amazon_description" ><small>-- ' . $this->eh_amazon_pay_options['pay_with_amazon_description'] . ' --</small></div>';
            $pwa_button_output .= "<div id='eh_pay_with_amazon' style='padding-bottom:10px;'></div>";
            $pwa_button_output .= "<div><small>-- or --</small></div></div></center>";
            echo $pwa_button_output;
        }
    }

    public function add_amazon_address_widget_section() {
        if ($this->eh_amazon && ($this->id !== '' || $this->token !== '' )) {
            ?>
            <div class="eh-amazon-main-widgets-section" id="eh_customer_widgets">
                <div class="col2-set">
                    <h3><?php _e('Shipping Address', 'eh-amazon-payments'); ?></h3>
                    <div id="amazon_address_widget_section"></div>
            <?php if (!empty($this->id)) { ?>
                        <input type="hidden" name="eh_amazon_payments_reference_id" value="<?php echo esc_attr($this->id); ?>" />
            <?php } elseif (!empty($this->token)) { ?>
                        <input type="hidden" name="eh_amazon_payments_access_token" value="<?php echo esc_attr($this->token); ?>" />
            <?php } ?>
                </div>
            <?php
        }
    }

    public function add_amazon_payment_widget_section() {
        if ($this->eh_amazon && ($this->id !== '' || $this->token !== '' )) {
            ?>
                <div class="col2-set">
                    <h3><?php _e('Payment Method', 'eh-amazon-payments'); ?></h3>
                    <div id="amazon_payment_widget_section"></div>
                <?php if (!empty($this->id)) { ?>
                        <input type="hidden" name="eh_amazon_payments_reference_id" value="<?php echo esc_attr($this->id); ?>" />
                <?php } elseif (!empty($this->token)) { ?>
                        <input type="hidden" name="eh_amazon_payments_access_token" value="<?php echo esc_attr($this->token); ?>" />
                <?php } ?>
                </div>
            <?php 
                $checkout = wc()->checkout;
                if ( ! is_user_logged_in() && $checkout->enable_signup ) 
                { 
                    if ( $checkout->enable_guest_checkout ) 
                    { ?>
                        <p class="form-row form-row-wide create-account">
                            <input class="input-checkbox" id="createaccount" <?php checked( ( true === $checkout->get_value( 'createaccount' ) || ( true === apply_filters( 'woocommerce_create_account_default_checked', false ) ) ), true ) ?> type="checkbox" name="createaccount" value="1" /> <label for="createaccount" class="checkbox"><?php _e( 'Create an account?', 'eh-amazon-payments' ); ?></label>
                        </p><?php 
                    }
                    do_action( 'woocommerce_before_checkout_registration_form', $checkout );
                    if ( ! empty( $checkout->checkout_fields['account'] ) ) 
                    { ?>
                        <div class="create-account">
                                <h3><?php _e( 'Create Account', 'eh-amazon-payments' ); ?></h3>
                                <p><?php _e( 'Create an account by entering the information below. If you are a returning customer please login at the top of the page.', 'eh-amazon-payments' ); ?></p>
                                <?php foreach ( $checkout->checkout_fields['account'] as $key => $field ) {
                                        woocommerce_form_field( $key, $field, $checkout->get_value( $key ) );
                                } ?>
                                <div class="clear"></div>
                        </div><?php 
                    }
                    do_action( 'woocommerce_after_checkout_registration_form', $checkout );
                } ?>
            </div>
            <?php
        }
    }
    public function remove_checkout_fields( $checkout ) {
        if ($this->eh_amazon && ($this->id !== '' || $this->token !== '' )) {
            $has_account_fields = (
                isset( $checkout->checkout_fields['account'] )
                &&
                is_array( $checkout->checkout_fields['account'] )
            );

            $has_billing_fields = (
                    isset( $checkout->checkout_fields['billing'] )
                    &&
                    is_array( $checkout->checkout_fields['billing'] )
            );

            if ( $has_account_fields && $has_billing_fields ) {
                    $billing_fields_to_copy = array(
                            'billing_first_name' => '',
                            'billing_last_name'  => '',
                            'billing_email'      => '',
                    );

                    $billing_fields_to_merge = array_intersect_key( $checkout->checkout_fields['billing'], $billing_fields_to_copy );

                    $checkout->checkout_fields['account'] = array_merge( $billing_fields_to_merge, $checkout->checkout_fields['account'] );

                    if ( isset( $checkout->checkout_fields['account']['billing_email']['class'] ) ) {
                            $checkout->checkout_fields['account']['billing_email']['class'] = '';
                    }
            }
            $checkout->checkout_fields['billing']  = array();
            $checkout->checkout_fields['shipping'] = array();
        }
    }
    public function gateways_hide_on_review($gateways) {
        if ($this->eh_amazon && ($this->id !== '' || $this->token !== '' )) {
            foreach ($gateways as $id => $name) {
                if ($id !== 'eh_amazon_pay') {
                    unset($gateways[$id]);
                }
            }
            return $gateways;
        }
        return $gateways;
    }

    public function add_policy_notes() {
        if ($this->eh_amazon && ($this->id !== '' || $this->token !== '' )) {
            echo
            '
                <div class="eh_amazon_seller_policy">
                    <div class="form-row eh_amazon_seller_policy_content">
                        <h3>Seller Policy</h3>
                        <div>' . $this->eh_amazon_pay_options['policy_notes'] . '</div>
                    </div>
                </div>
            ';
        }
    }

    public function add_cancel_order_elements() {
        if ($this->eh_amazon && ($this->id !== '' || $this->token !== '' )) {
            printf('<a href="' . add_query_arg('cancel_amazon_checkout', 'true', wc_get_checkout_url()) . '" class="button alt " id="cancel_amazon_order" style="background-color: crimson;">Cancel Order</a>');
        }
    }

    public function update_amazon_widgets_fragment($fragments) {
        $available_gateways = WC()->payment_gateways()->get_available_payment_gateways();

        if (WC()->cart->needs_payment()) {

            if (array_key_exists('eh_amazon_pay', $available_gateways)) {

                ob_start();

                $this->add_amazon_address_widget_section();

                $this->add_amazon_payment_widget_section();

                $fragments['#eh_customer_widgets'] = ob_get_clean();
            }
        }
        return $fragments;
    }

    public function update_check_info() {
        if ($this->eh_amazon) {
            if (!$this->id) {
                return;
            }
            $args = array();
            $args['amazon_order_reference_id'] = $this->id;
            if ($this->eh_amazon_pay_options['ssl_mode'] === 'https') {
                $args['mws_auth_token'] = $this->token;
            }
            $amazon_req = new Eh_Amazon_Request_Response();
            $data_object = $amazon_req->request_amazon('GetOrderReferenceDetails', $args);
            $data = $data_object->GetOrderReferenceDetailsResult->OrderReferenceDetails;
            if (!$data || !isset($data->Destination->PhysicalDestination)) {
                return;
            }
            $customer_address = (array) $data->Destination->PhysicalDestination;
            if (!empty($customer_address['CountryCode'])) {
                WC()->customer->set_country($customer_address['CountryCode']);
                WC()->customer->set_shipping_country($customer_address['CountryCode']);
            }

            if (!empty($customer_address['StateOrRegion'])) {
                WC()->customer->set_state($customer_address['StateOrRegion']);
                WC()->customer->set_shipping_state($customer_address['StateOrRegion']);
            }

            if (!empty($customer_address['PostalCode'])) {

                WC()->customer->set_postcode($customer_address['PostalCode']);
                WC()->customer->set_shipping_postcode($customer_address['PostalCode']);
            }

            if (!empty($customer_address['City'])) {
                WC()->customer->set_city($customer_address['City']);
                WC()->customer->set_shipping_city($customer_address['City']);
            }
        }
    }
}
